from collections import defaultdict, Counter
from dataclasses import dataclass
from enum import Enum
from random import shuffle, randrange
from typing import List, Dict

from model.animals import random_animal_name


class Value(Enum):
    Two = 1
    Three = 2
    Four = 3
    Five = 4
    Six = 5
    Seven = 6
    Eight = 7
    Nine = 8
    Ten = 9
    Jack = 10
    Queen = 11
    King = 12
    Ace = 13


class Color(Enum):
    Hearts = "♥"
    Spades = "♠"
    Clubs = "♣"
    Diamonds = "♦"


@dataclass(frozen=True)
class Card:
    value: Value
    color: Color

    def __str__(self) -> str:
        return f"{self.color.value} {self.value.name} {self.color.value}"


class Hand:
    def __init__(self, cards=None):
        if cards is None:
            cards = []
        self.cards: List[Card] = cards

    def __len__(self):
        return len(self.cards)

    def __getitem__(self, item):
        return self.cards[item]

    def give(self, other: Card):
        self.cards.append(other)

    def value(self):
        print(f"Looking at value of {len(self.cards)} cards")
        counter = Counter(self.cards)
        print(counter)
        if len(self.cards) == 1: # Simple, v=x
            return int(self.cards[0].value.value)

        # elif len(self.cards) == 2: # Paire, v=20*x
        # elif len(self.cards) == 3: # Brelan ou paire, v=20*x
        # elif len(self.cards) == 2: # Paire, v=20*x
        # elif len(self.cards) == 2: # Paire, v=20*x


class Deck:
    def __init__(self):
        self.cards = [Card(v, c) for v in Value for c in Color]
        self.defausse = []

    def __len__(self):
        return len(self.cards)

    def random_card(self) -> Card:
        if not self.cards:
            self.reset()

        card = self.cards.pop(randrange(len(self.cards)))
        self.defausse.append(card)
        return card

    def reset(self):
        self.cards.extend(self.defausse)
        self.defausse.clear()
        shuffle(self.cards)


class Player:
    name: str = random_animal_name()
    hand: Hand = Hand()

    def give(self, card: Card):
        self.hand.give(card)


class Game:
    def __init__(self, deck: Deck = Deck(), players=None):
        if players is None:
            players = []

        self.deck: Deck = deck
        self.players: List[Player] = players
        self.defeats: Dict[Player, int] = defaultdict(lambda: 0)

    def new_game(self):
        self.deck.reset()
        while len(self.players) > 1:
            self.new_turn()
        winner = self.players[0]
        print(f"Game over - Player {winner.name} wins with {len(winner.hand)} cards!")
        pass

    def new_turn(self):
        # Distribution
        shuffle(self.deck)
        for p in self.players:
            p.give(self.deck.random_card())

        # Tour
        for p in self.players:
            announce = p.announce(current_bet)

        # TODO: Put next first player first of list
